/*!
 * @asigloo/vue-dynamic-forms v1.x.x
 * (c) 2020 Alvaro Saburido
 * @license MIT
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');

const dynamicFormsSymbol = Symbol();
function useDynamicForms() {
    const dynamicForms = vue.inject(dynamicFormsSymbol);
    if (!dynamicForms)
        throw new Error('No dynamicForms provided!!!');
    return dynamicForms;
}

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function createCommonjsModule(fn, basedir, module) {
	return module = {
		path: basedir,
		exports: {},
		require: function (path, base) {
			return commonjsRequire(path, (base === undefined || base === null) ? module.path : base);
		}
	}, fn(module, module.exports), module.exports;
}

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
}

var utils = createCommonjsModule(function (module, exports) {
(function (global, factory) {
  {
    factory(exports);
  }
})(commonjsGlobal, function (exports) {

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
    return typeof obj;
  } : function (obj) {
    return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
  };

  var isDate = exports.isDate = function isDate(d) {
    return d instanceof Date;
  };
  var isEmpty = exports.isEmpty = function isEmpty(o) {
    return Object.keys(o).length === 0;
  };
  var isObject = exports.isObject = function isObject(o) {
    return o != null && (typeof o === 'undefined' ? 'undefined' : _typeof(o)) === 'object';
  };
  var properObject = exports.properObject = function properObject(o) {
    return isObject(o) && !o.hasOwnProperty ? _extends({}, o) : o;
  };
});
});

var diff = createCommonjsModule(function (module, exports) {
(function (global, factory) {
  {
    factory(module, exports, utils);
  }
})(commonjsGlobal, function (module, exports, _utils) {

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var diff = function diff(lhs, rhs) {
    if (lhs === rhs) return {}; // equal return no diff

    if (!(0, _utils.isObject)(lhs) || !(0, _utils.isObject)(rhs)) return rhs; // return updated rhs

    var l = (0, _utils.properObject)(lhs);
    var r = (0, _utils.properObject)(rhs);

    var deletedValues = Object.keys(l).reduce(function (acc, key) {
      return r.hasOwnProperty(key) ? acc : _extends({}, acc, _defineProperty({}, key, undefined));
    }, {});

    if ((0, _utils.isDate)(l) || (0, _utils.isDate)(r)) {
      if (l.valueOf() == r.valueOf()) return {};
      return r;
    }

    return Object.keys(r).reduce(function (acc, key) {
      if (!l.hasOwnProperty(key)) return _extends({}, acc, _defineProperty({}, key, r[key])); // return added r key

      var difference = diff(l[key], r[key]);

      if ((0, _utils.isObject)(difference) && (0, _utils.isEmpty)(difference) && !(0, _utils.isDate)(difference)) return acc; // return no diff

      return _extends({}, acc, _defineProperty({}, key, difference)); // return updated key
    }, deletedValues);
  };

  exports.default = diff;
  module.exports = exports['default'];
});
});

var added = createCommonjsModule(function (module, exports) {
(function (global, factory) {
  {
    factory(module, exports, utils);
  }
})(commonjsGlobal, function (module, exports, _utils) {

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var addedDiff = function addedDiff(lhs, rhs) {

    if (lhs === rhs || !(0, _utils.isObject)(lhs) || !(0, _utils.isObject)(rhs)) return {};

    var l = (0, _utils.properObject)(lhs);
    var r = (0, _utils.properObject)(rhs);

    return Object.keys(r).reduce(function (acc, key) {
      if (l.hasOwnProperty(key)) {
        var difference = addedDiff(l[key], r[key]);

        if ((0, _utils.isObject)(difference) && (0, _utils.isEmpty)(difference)) return acc;

        return _extends({}, acc, _defineProperty({}, key, difference));
      }

      return _extends({}, acc, _defineProperty({}, key, r[key]));
    }, {});
  };

  exports.default = addedDiff;
  module.exports = exports['default'];
});
});

var deleted = createCommonjsModule(function (module, exports) {
(function (global, factory) {
  {
    factory(module, exports, utils);
  }
})(commonjsGlobal, function (module, exports, _utils) {

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var deletedDiff = function deletedDiff(lhs, rhs) {
    if (lhs === rhs || !(0, _utils.isObject)(lhs) || !(0, _utils.isObject)(rhs)) return {};

    var l = (0, _utils.properObject)(lhs);
    var r = (0, _utils.properObject)(rhs);

    return Object.keys(l).reduce(function (acc, key) {
      if (r.hasOwnProperty(key)) {
        var difference = deletedDiff(l[key], r[key]);

        if ((0, _utils.isObject)(difference) && (0, _utils.isEmpty)(difference)) return acc;

        return _extends({}, acc, _defineProperty({}, key, difference));
      }

      return _extends({}, acc, _defineProperty({}, key, undefined));
    }, {});
  };

  exports.default = deletedDiff;
  module.exports = exports['default'];
});
});

var updated = createCommonjsModule(function (module, exports) {
(function (global, factory) {
  {
    factory(module, exports, utils);
  }
})(commonjsGlobal, function (module, exports, _utils) {

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var updatedDiff = function updatedDiff(lhs, rhs) {

    if (lhs === rhs) return {};

    if (!(0, _utils.isObject)(lhs) || !(0, _utils.isObject)(rhs)) return rhs;

    var l = (0, _utils.properObject)(lhs);
    var r = (0, _utils.properObject)(rhs);

    if ((0, _utils.isDate)(l) || (0, _utils.isDate)(r)) {
      if (l.valueOf() == r.valueOf()) return {};
      return r;
    }

    return Object.keys(r).reduce(function (acc, key) {

      if (l.hasOwnProperty(key)) {
        var difference = updatedDiff(l[key], r[key]);

        if ((0, _utils.isObject)(difference) && (0, _utils.isEmpty)(difference) && !(0, _utils.isDate)(difference)) return acc;

        return _extends({}, acc, _defineProperty({}, key, difference));
      }

      return acc;
    }, {});
  };

  exports.default = updatedDiff;
  module.exports = exports['default'];
});
});

var detailed = createCommonjsModule(function (module, exports) {
(function (global, factory) {
  {
    factory(module, exports, added, deleted, updated);
  }
})(commonjsGlobal, function (module, exports, _added, _deleted, _updated) {

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _added2 = _interopRequireDefault(_added);

  var _deleted2 = _interopRequireDefault(_deleted);

  var _updated2 = _interopRequireDefault(_updated);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  var detailedDiff = function detailedDiff(lhs, rhs) {
    return {
      added: (0, _added2.default)(lhs, rhs),
      deleted: (0, _deleted2.default)(lhs, rhs),
      updated: (0, _updated2.default)(lhs, rhs)
    };
  };

  exports.default = detailedDiff;
  module.exports = exports['default'];
});
});

var dist = createCommonjsModule(function (module, exports) {
(function (global, factory) {
  {
    factory(exports, diff, added, deleted, updated, detailed);
  }
})(commonjsGlobal, function (exports, _diff, _added, _deleted, _updated, _detailed) {

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.detailedDiff = exports.updatedDiff = exports.deletedDiff = exports.diff = exports.addedDiff = undefined;

  var _diff2 = _interopRequireDefault(_diff);

  var _added2 = _interopRequireDefault(_added);

  var _deleted2 = _interopRequireDefault(_deleted);

  var _updated2 = _interopRequireDefault(_updated);

  var _detailed2 = _interopRequireDefault(_detailed);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  exports.addedDiff = _added2.default;
  exports.diff = _diff2.default;
  exports.deletedDiff = _deleted2.default;
  exports.updatedDiff = _updated2.default;
  exports.detailedDiff = _detailed2.default;
});
});

/* eslint-disable */
const isArray = (a) => !!a && a.constructor === Array;
const isObject = (a) => !!a && a.constructor === Object;
const hasValue = (value) => value !== undefined && value !== null && value !== '';
const removeEmpty = obj => Object.keys(obj)
    .filter(k => obj[k] != null) // Remove undef. and null.
    .reduce((newObj, k) => typeof obj[k] === 'object'
    ? { ...newObj, [k]: removeEmpty(obj[k]) } // Recurse.
    : { ...newObj, [k]: obj[k] }, // Copy value.
{});
const deepClone = (obj) => {
    let clone, value, key;
    if (typeof obj !== "object" || obj === null) {
        return obj; // Return the value if obj is not an object
    }
    // Create an array or object to hold the values
    clone = Array.isArray(obj) ? [] : {};
    for (key in obj) {
        value = obj[key];
        // Recursively (deep) copy for nested objects, including arrays
        clone[key] = deepClone(value);
    }
    return clone;
};

/* eslint-disable */
function useInputValidation(props, emit) {
    const isPendingValidation = vue.ref(false);
    const isRequired = vue.computed(() => {
        return props.control.validations.some(validation => validation.type === 'required');
    });
    const requiresValidation = vue.computed(() => {
        return props.control.validations.length > 0;
    });
    async function validate(force = false) {
        if (force || (props.control.touched || props.control.dirty) &&
            requiresValidation.value) {
            let errors = {};
            const syncValidations = [];
            const asyncValidations = [];
            props.control.validations.forEach(element => {
                const validation = element.validator(props.control.value);
                if (validation.constructor.name === 'Promise') {
                    asyncValidations.push({
                        validation: element.validator,
                        text: element.text,
                    });
                }
                else {
                    syncValidations.push({ validation, text: element.text });
                }
            });
            if (asyncValidations.length > 0) {
                isPendingValidation.value = true;
                Promise.all(asyncValidations.map(async ({ validation, text }) => ({
                    validation: await validation(props.control.value),
                    text,
                }))).then(errorsArr => {
                    errorsArr.forEach(({ validation, text }) => {
                        const [key, value] = Object.entries(validation)[0];
                        errors[key] = value
                            ? {
                                value,
                                text,
                            }
                            : null;
                    });
                    isPendingValidation.value = false;
                    emit('validate', {
                        name: props.control.name,
                        errors,
                        valid: Object.keys(removeEmpty(errors)).length === 0,
                    });
                });
            }
            syncValidations.forEach(({ validation, text }) => {
                if (validation) {
                    const [key, value] = Object.entries(validation)[0];
                    errors[key] = value
                        ? {
                            value,
                            text,
                        }
                        : null;
                }
            });
            emit('validate', {
                name: props.control.name,
                errors,
                valid: Object.keys(removeEmpty(errors)).length === 0,
            });
        }
    }
    const errorMessages = vue.computed(() => {
        var _a;
        const errors = Object.values(((_a = props.control) === null || _a === void 0 ? void 0 : _a.errors) || {});
        if (errors.length > 0) {
            return errors.map((error) => error.text);
        }
        return [];
    });
    const getValidationClasses = vue.computed(() => {
        return [
            {
                'form-control--success': !isPendingValidation.value &&
                    requiresValidation.value &&
                    props.control.errors &&
                    props.control.valid &&
                    props.control.dirty &&
                    props.control.touched,
            },
            {
                'form-control--error': !isPendingValidation.value && !props.control.valid,
                'form-control--validating': isPendingValidation.value
            },
        ];
    });
    vue.watch(() => props.forceValidation, value => {
        if (value) {
            validate(value);
        }
    });
    return {
        isPendingValidation,
        validate,
        getValidationClasses,
        isRequired,
        requiresValidation,
        errorMessages,
    };
}

/* eslint-disable @typescript-eslint/explicit-module-boundary-types */
function useInputEvents(props, emit) {
    const { validate, getValidationClasses } = useInputValidation(props, emit);
    function onInput($event) {
        const element = $event.target;
        $event.stopImmediatePropagation();
        if (props.control && hasValue(element.value)) {
            if ((!props.control.valid &&
                props.control.validationTrigger.type ===
                    "blur" /* BLUR */) ||
                (props.control.validationTrigger.type ===
                    "change" /* CHANGE */ &&
                    element.value.length >= props.control.validationTrigger.threshold)) {
                validate();
            }
            emit('change', {
                name: props.control.name,
                value: element.value,
            });
        }
    }
    function onChange($event) {
        $event.stopImmediatePropagation();
        $event.preventDefault();
    }
    function onCheck($event) {
        const element = $event.target;
        if (props.control) {
            $event.stopImmediatePropagation();
            emit('change', {
                name: props.control.name,
                value: element.checked,
            });
        }
    }
    function onFocus() {
        emit('focus', { name: props.control.name });
    }
    function onBlur() {
        emit('blur', { name: props.control.name });
        if (props.control.validationTrigger.type === "blur" /* BLUR */) {
            validate();
        }
    }
    const getClasses = vue.computed(() => {
        return ['form-control', ...getValidationClasses.value];
    });
    vue.watch(() => { var _a; return (_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.value; }, (curr, prev) => {
        if (hasValue(curr) && curr !== prev) {
            emit('change', {
                name: props.control.name,
                value: props.control.value,
            });
        }
    }, {
        immediate: true,
    });
    return {
        onFocus,
        onInput,
        onChange,
        onBlur,
        onCheck,
        getClasses,
    };
}

const props = {
    control: Object,
    forceValidation: {
        type: Boolean,
        default: false,
    },
};
var script = vue.defineComponent({
    name: 'asTextInput',
    inheritAttrs: false,
    props,
    setup(props, { emit }) {
        const { onInput, onChange, onFocus, onBlur, getClasses } = useInputEvents(props, emit);
        const { isRequired, errorMessages, isPendingValidation, } = useInputValidation(props, emit);
        return () => [
            vue.h('input', {
                id: props.control.name,
                name: props.control.name || '',
                type: props.control.type,
                class: getClasses.value,
                value: props.control.value,
                disabled: props.control.disabled,
                placeholder: props.control.placeholder,
                required: isRequired.value,
                readonly: props.control.readonly,
                autocomplete: props.control.autocomplete,
                ariaRequired: isRequired.value,
                ariaLabel: props.control.ariaLabel,
                ariaLabelledBy: props.control.ariaLabelledBy,
                onFocus,
                onBlur,
                onInput,
                onChange,
            }),
            isPendingValidation.value
                ? null
                : vue.h('div', {
                    class: 'form-errors',
                }, errorMessages.value.map(error => vue.h('p', { class: 'form-error' }, error))),
        ];
    },
});

script.__file = "src/components/text-input/TextInput.vue";

const props$1 = {
    control: Object,
    forceValidation: {
        type: Boolean,
        default: false,
    },
};
var script$1 = vue.defineComponent({
    name: 'asSelectInput',
    inheritAttrs: false,
    props: props$1,
    setup(props, { emit }) {
        return () => {
            var _a, _b, _c, _d;
            const { onInput, onChange, onFocus, onBlur } = useInputEvents(props, emit);
            const { isRequired, errorMessages, isPendingValidation, } = useInputValidation(props, emit);
            const formattedOptions = vue.computed(() => {
                var _a, _b, _c;
                if (isObject((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.options)) {
                    return Object.values((_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.options);
                }
                return (_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.options;
            });
            const options = formattedOptions.value.map(option => vue.h('option', {
                key: option[props.control.optionValue],
                value: option[props.control.optionValue],
                disabled: option.disabled,
            }, option[props.control.optionLabel]));
            return [
                vue.h('select', {
                    id: props.control.name,
                    name: ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.name) || '',
                    class: ['form-control'],
                    value: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.value,
                    disabled: (_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.disabled,
                    placeholder: (_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.placeholder,
                    required: isRequired.value,
                    readonly: props === null || props === void 0 ? void 0 : props.control.readonly,
                    ariaLabel: props.control.ariaLabel,
                    ariaLabelledBy: props.control.ariaLabelledBy,
                    ariaRequired: isRequired.value,
                    onFocus,
                    onBlur,
                    onInput,
                    onChange,
                }, options),
                isPendingValidation.value
                    ? null
                    : vue.h('div', {
                        class: 'form-errors',
                    }, errorMessages.value.map(error => vue.h('p', { class: 'form-error' }, error))),
            ];
        };
    },
});

script$1.__file = "src/components/select-input/SelectInput.vue";

const props$2 = {
    control: Object,
    forceValidation: {
        type: Boolean,
        default: false,
    },
};
var script$2 = vue.defineComponent({
    name: 'asTextAreaInput',
    inheritAttrs: false,
    props: props$2,
    setup(props, { emit }) {
        const { onInput, onChange, onFocus, onBlur } = useInputEvents(props, emit);
        const { isRequired, errorMessages, isPendingValidation, } = useInputValidation(props, emit);
        return () => [
            vue.h('textarea', {
                id: props.control.name,
                name: props.control.name || '',
                class: ['form-control'],
                value: props.control.value,
                rows: props.control.rows,
                cols: props.control.cols,
                disabled: props.control.disabled,
                placeholder: props.control.placeholder,
                required: isRequired.value,
                autocomplete: props.control.autocomplete,
                readonly: props.control.readonly,
                ariaLabel: props.control.ariaLabel,
                ariaLabelledBy: props.control.ariaLabelledBy,
                ariaRequired: isRequired.value,
                onFocus,
                onBlur,
                onInput,
                onChange,
            }),
            isPendingValidation.value
                ? null
                : vue.h('div', {
                    class: 'form-errors',
                }, errorMessages.value.map(error => vue.h('p', { class: 'form-error' }, error))),
        ];
    },
});

script$2.__file = "src/components/text-area-input/TextAreaInput.vue";

const props$3 = {
    control: Object,
    forceValidation: {
        type: Boolean,
        default: false,
    },
};
var script$3 = vue.defineComponent({
    name: 'asCheckboxInput',
    inheritAttrs: false,
    props: props$3,
    setup(props, { emit }) {
        const { onCheck, onFocus, onBlur } = useInputEvents(props, emit);
        const { errorMessages, isPendingValidation } = useInputValidation(props, emit);
        const renderCheckbox = [
            vue.h('input', {
                name: props.control.name || '',
                type: props.control.type,
                id: props.control.name,
                disabled: props.control.disabled,
                class: ['checkbox-control'],
                value: props.control.value,
                checked: props.control.value,
                onFocus,
                onBlur,
                onChange: onCheck,
            }),
            vue.h('label', {
                class: ['checkbox-label'],
                for: props.control.name,
            }, props.control.label),
        ];
        return () => [
            vue.h('div', {
                class: 'checkbox-group',
                tabIndex: -1,
                role: 'group',
            }, renderCheckbox),
            isPendingValidation.value
                ? null
                : vue.h('div', {
                    class: 'form-errors',
                }, errorMessages.value.map(error => vue.h('p', { class: 'form-error' }, error))),
        ];
    },
});

script$3.__file = "src/components/checkbox-input/CheckboxInput.vue";

const props$4 = {
    control: Object,
    forceValidation: {
        type: Boolean,
        default: false,
    },
};
var script$4 = vue.defineComponent({
    name: 'asRadioInput',
    inheritAttrs: false,
    props: props$4,
    setup(props, { emit }) {
        var _a, _b;
        const { onChange, onInput, onFocus, onBlur } = useInputEvents(props, emit);
        const { errorMessages, isPendingValidation } = useInputValidation(props, emit);
        const renderRadios = (_b = (_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.map(option => {
            var _a, _b;
            return vue.h('div', { class: 'radio-input' }, [
                vue.h('input', {
                    name: ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.name) || '',
                    type: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.type,
                    id: option.key,
                    disabled: option.disabled,
                    class: ['radio-control'],
                    value: option.value,
                    onFocus,
                    onBlur,
                    onInput,
                    onChange,
                }),
                vue.h('label', {
                    class: ['radio-label'],
                    for: option.key,
                }, option.value),
            ]);
        });
        return () => [
            vue.h('div', {
                class: 'radio-group',
                tabIndex: -1,
                role: 'group',
            }, renderRadios),
            isPendingValidation.value
                ? null
                : vue.h('div', {
                    class: 'form-errors',
                }, errorMessages.value.map(error => vue.h('p', { class: 'form-error' }, error))),
        ];
    },
});

script$4.__file = "src/components/radio-input/RadioInput.vue";

const props$5 = {
    control: Object,
    forceValidation: {
        type: Boolean,
        default: false,
    },
};
var script$5 = vue.defineComponent({
    name: 'asNumberInput',
    inheritAttrs: false,
    props: props$5,
    setup(props, { emit }) {
        const { onInput, onChange, onFocus, onBlur } = useInputEvents(props, emit);
        const { isRequired, errorMessages, isPendingValidation, } = useInputValidation(props, emit);
        return () => [
            vue.h('input', {
                id: props.control.name,
                name: props.control.name || '',
                type: props.control.type,
                class: ['form-control'],
                value: props.control.value,
                min: props.control.min,
                max: props.control.max,
                step: props.control.step,
                disabled: props.control.disabled,
                placeholder: props.control.placeholder,
                required: isRequired.value,
                readonly: props.control.readonly,
                autocomplete: props.control.autocomplete,
                ariaLabel: props.control.ariaLabel,
                ariaLabelledBy: props.control.ariaLabelledBy,
                ariaRequired: isRequired.value,
                onFocus,
                onBlur,
                onInput,
                onChange,
            }),
            isPendingValidation.value
                ? null
                : vue.h('div', {
                    class: 'form-errors',
                }, errorMessages.value.map(error => vue.h('p', { class: 'form-error' }, error))),
        ];
    },
});

script$5.__file = "src/components/number-input/NumberInput.vue";

/* eslint-disable @typescript-eslint/no-use-before-define */
const components = {
    TextInputComponent: script,
    SelectInputComponent: script$1,
    TextAreaInputComponent: script$2,
    CheckboxInputComponent: script$3,
    RadioInputComponent: script$4,
    NumberInputComponent: script$5,
};
const props$6 = {
    control: {
        type: Object,
        required: true,
    },
    forceValidation: {
        type: Boolean,
        default: false,
    },
};
var script$6 = vue.defineComponent({
    name: 'asDynamicInput',
    components,
    inheritAttrs: false,
    props: props$6,
    setup(props, { emit, slots }) {
        const { onFocus, onInput, onBlur } = useInputEvents(props, emit);
        let component;
        const attributes = vue.computed(() => {
            return {
                control: props === null || props === void 0 ? void 0 : props.control,
                style: props === null || props === void 0 ? void 0 : props.control.customStyles,
                onChange: valueChange,
                onBlur: (e) => emit('blur', e),
                onFocus: (e) => emit('focus', e),
                onValidate: (validation) => emit('validate', validation),
                forceValidation: props.forceValidation,
            };
        });
        const hasLabel = vue.computed(() => { var _a, _b; return ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.label) && ((_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.type) !== 'checkbox'; });
        const isFieldSet = vue.computed(() => { var _a; return ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) === 'radio'; });
        const getClasses = vue.computed(() => {
            var _a, _b, _c, _d, _e, _f;
            const classes = [
                'dynamic-input',
                'form-group',
                {
                    'form-group--inline': ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) === "checkbox" /* CHECKBOX */,
                },
            ];
            if (isArray((_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.customClass)) {
                return [
                    ...classes,
                    ...(_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.customClass,
                ];
            }
            if (isObject((_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.customClass)) {
                return [...classes, (_e = props === null || props === void 0 ? void 0 : props.control) === null || _e === void 0 ? void 0 : _e.customClass];
            }
            return [classes, (_f = props === null || props === void 0 ? void 0 : props.control) === null || _f === void 0 ? void 0 : _f.customClass];
        });
        function valueChange(event) {
            emit('change', event);
        }
        return () => {
            var _a, _b, _c, _d, _e;
            switch ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) {
                case "text" /* TEXT */:
                    component = vue.h(script, attributes.value);
                    break;
                case 'email':
                    component = vue.h(script, attributes.value);
                    break;
                case 'password':
                    component = vue.h(script, attributes.value);
                    break;
                case 'url':
                    component = vue.h(script, attributes.value);
                    break;
                case 'color':
                    component = vue.h(script, attributes.value);
                    break;
                case 'number':
                    component = vue.h(script$5, attributes.value);
                    break;
                case 'select':
                    component = vue.h(script$1, attributes.value);
                    break;
                case 'textarea':
                    component = vue.h(script$2, attributes.value);
                    break;
                case 'checkbox':
                    component = vue.h(script$3, attributes.value);
                    break;
                case 'radio':
                    component = vue.h(script$4, attributes.value);
                    break;
                case 'custom-field':
                    component = vue.h('slot', {
                        name: 'customField',
                    }, slots.customField({
                        control: props.control,
                        onChange: onInput,
                        onFocus,
                        onBlur,
                    }));
                    break;
            }
            const requiredStar = vue.h('span', {
                ariaHidden: true,
                class: 'form-required-star',
            }, ' *');
            return vue.h(isFieldSet.value ? 'fieldset' : 'div', {
                class: getClasses.value,
                role: isFieldSet.value ? undefined : 'group',
            }, [
                hasLabel.value
                    ? vue.h(isFieldSet.value ? 'legend' : 'label', {
                        class: 'form-label',
                        for: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.name,
                    }, [
                        `${(_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.label}`,
                        ((_e = (_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.validations) === null || _e === void 0 ? void 0 : _e.some(validator => validator.type === 'required')) ? requiredStar
                            : '',
                    ])
                    : null,
                component,
            ]);
        };
    },
});

script$6.__file = "src/components/dynamic-input/DynamicInput.vue";

const EMPTY_CONTROL = {
    dirty: false,
    touched: false,
    valid: true,
    errors: null,
};
const FieldBase = ({ validations = [], label = null, ariaLabel = null, ariaLabelledBy = null, customClass = null, customStyles = null, disabled = false, placeholder = null, autocomplete = null, readonly = false, validationTrigger = ValidatorTrigger({
    type: "blur" /* BLUR */,
    threshold: 0,
}), }) => ({
    validations,
    label,
    ariaLabel,
    ariaLabelledBy,
    customClass,
    customStyles,
    disabled,
    placeholder,
    autocomplete,
    readonly,
    validationTrigger,
});
const TextField = ({ value, ...rest }) => ({
    ...FieldBase(rest),
    value,
    type: "text" /* TEXT */,
});
const TextAreaField = ({ value, cols, rows, ...rest }) => ({
    ...FieldBase(rest),
    value,
    cols,
    rows,
    type: "textarea" /* TEXTAREA */,
});
const EmailField = ({ value, ...rest }) => ({
    ...FieldBase(rest),
    value,
    type: "email" /* EMAIL */,
});
const PasswordField = ({ value, ...rest }) => ({
    ...FieldBase(rest),
    value,
    type: "password" /* PASSWORD */,
});
const CheckboxField = ({ value, ...rest }) => ({
    ...FieldBase(rest),
    value,
    type: "checkbox" /* CHECKBOX */,
});
const ColorField = ({ value, ...rest }) => ({
    ...FieldBase(rest),
    value,
    type: "color" /* COLOR */,
});
const RadioField = ({ options, value, ...rest }) => ({
    ...FieldBase(rest),
    value,
    options,
    type: "radio" /* RADIO */,
});
const NumberField = ({ value = null, min = 0, max = 100, step = 1, ...rest }) => ({
    ...FieldBase(rest),
    value,
    min,
    max,
    step,
    type: "number" /* NUMBER */,
});
const SelectField = ({ options = [], value, optionValue = 'value', optionLabel = 'label', ...rest }) => ({
    ...FieldBase(rest),
    value,
    options,
    optionValue,
    optionLabel,
    type: "select" /* SELECT */,
});
const CustomField = ({ value, ...rest }) => ({
    ...FieldBase(rest),
    value,
    type: "custom-field" /* CUSTOM */,
});
const FieldControl = ({ name, type, ...rest }) => ({
    ...rest,
    name,
    type,
    ...EMPTY_CONTROL,
});
const Validator = ({ validator, text, }) => ({
    type: validator(undefined) ? Object.keys(validator(undefined))[0] : 'pattern',
    validator,
    text,
});
const ValidatorTrigger = ({ type, threshold, }) => ({
    type,
    threshold,
});

/**
 * @internal
 */
function createFilterWrapper(filter, fn) {
    function wrapper(...args) {
        filter(() => fn.apply(this, args), { fn, thisArg: this, args });
    }
    return wrapper;
}
const bypassFilter = invoke => {
    return invoke();
};
function debounceFilter(ms) {
    if (ms <= 0)
        return bypassFilter;
    let timer;
    const filter = invoke => {
        if (timer)
            clearTimeout(timer);
        timer = setTimeout(invoke, ms);
    };
    return filter;
}
function useDebounceFn(fn, ms = 200) {
    return createFilterWrapper(debounceFilter(ms), fn);
}

const props$7 = {
    form: {
        type: Object,
        required: true,
    },
};
const components$1 = {
    DynamicInput: script$6,
};
/* const AVAILABLE_THEMES = ['default', 'material'];
 */
var script$7 = vue.defineComponent({
    name: 'asDynamicForm',
    inheritAttrs: false,
    props: props$7,
    components: components$1,
    setup(props, ctx) {
        const { options } = vue.inject(dynamicFormsSymbol);
        let cache = deepClone(vue.toRaw(props.form.fields));
        const controls = vue.ref([]);
        const forceValidation = vue.ref(false);
        const deNormalizedScopedSlots = vue.computed(() => Object.keys(ctx.slots));
        const normalizedControls = vue.computed(() => {
            let normalizedControls = {};
            controls.value.forEach(element => {
                normalizedControls[element.name] = element;
            });
            return normalizedControls;
        });
        const isValid = vue.computed(() => {
            const hasInvalidControls = controls.value.some(control => !control.valid);
            return !hasInvalidControls;
        });
        const formValues = vue.computed(() => {
            return removeEmpty(controls.value.reduce((prev, curr) => {
                const obj = {};
                obj[curr.name] =
                    curr.type === "number" /* NUMBER */
                        ? parseFloat(`${curr.value}`)
                        : curr.value;
                return {
                    ...prev,
                    ...obj,
                };
            }, {}));
        });
        const errors = vue.computed(() => {
            return controls.value
                ? controls.value.reduce((prev, curr) => {
                    const errors = Object.keys(curr.errors || {}) || [];
                    if (errors.length > 0) {
                        const error = {};
                        error[curr.name] = curr.errors;
                        return {
                            ...prev,
                            ...error,
                        };
                    }
                    return prev;
                }, {})
                : {};
        });
        const formattedOptions = vue.computed(() => {
            if (options === null || options === void 0 ? void 0 : options.form) {
                const { customClass, customStyles, method, netlify, netlifyHoneypot, autocomplete, } = options === null || options === void 0 ? void 0 : options.form;
                return {
                    class: customClass,
                    style: customStyles,
                    method,
                    'data-netlify': netlify,
                    'data-netlify-honeypot': netlifyHoneypot,
                    autocomplete: autocomplete ? 'on' : 'off',
                };
            }
            else {
                return;
            }
        });
        function mapControls(empty = false) {
            var _a;
            const controlArray = Object.entries((_a = props.form) === null || _a === void 0 ? void 0 : _a.fields).map(([key, field]) => empty
                ? FieldControl({
                    ...field,
                    name: key,
                    value: null,
                })
                : FieldControl({
                    ...field,
                    name: key,
                })) || [];
            if (props.form.fieldOrder) {
                controls.value = controlArray.sort((a, b) => props.form.fieldOrder.indexOf(a.name) -
                    props.form.fieldOrder.indexOf(b.name));
            }
            else {
                controls.value = controlArray;
            }
        }
        function findControlByName(name) {
            const updatedCtrl = controls.value.find(control => control.name === name);
            return updatedCtrl;
        }
        function emitChanges(changes) {
            ctx.emit('change', changes);
        }
        const debounceEmitChanges = useDebounceFn(emitChanges, 300);
        function valueChange(event) {
            if (hasValue(event.value)) {
                const updatedCtrl = findControlByName(event.name);
                if (updatedCtrl) {
                    updatedCtrl.value = event.value;
                    updatedCtrl.dirty = true;
                }
                debounceEmitChanges(formValues.value);
            }
        }
        function onBlur({ name }) {
            const updatedCtrl = findControlByName(name);
            if (updatedCtrl) {
                updatedCtrl.touched = true;
            }
        }
        function onValidate({ name, errors, valid }) {
            const updatedCtrl = findControlByName(name);
            if (updatedCtrl) {
                updatedCtrl.errors = removeEmpty({
                    ...updatedCtrl.errors,
                    ...errors,
                });
                updatedCtrl.valid = valid;
            }
        }
        function detectChanges(fields) {
            const changes = dist.diff(cache, deepClone(fields));
            Object.entries(changes).forEach(([key, value]) => {
                let ctrl = findControlByName(key);
                if (ctrl) {
                    Object.entries(value).forEach(([change, newValue]) => {
                        if (change === 'options' || change === 'validations') {
                            Object.entries(newValue).forEach(([optKey, optValue]) => {
                                ctrl[change][optKey] = {
                                    ...ctrl[change][optKey],
                                    ...optValue,
                                };
                            });
                        }
                        else {
                            ctrl[change] = newValue;
                        }
                    });
                }
            });
            cache = deepClone(vue.toRaw(props.form.fields));
        }
        function resetForm() {
            mapControls(true);
            forceValidation.value = false;
        }
        async function handleSubmit() {
            validateAll();
            await vue.nextTick();
            if (isValid.value) {
                ctx.emit('submitted', formValues);
                resetForm();
            }
            else {
                ctx.emit('error', formValues);
            }
        }
        function validateAll() {
            forceValidation.value = true;
        }
        vue.watch(() => props.form.fields, fields => {
            detectChanges(fields);
        }, {
            deep: true,
        });
        vue.onMounted(() => {
            mapControls();
        });
        return {
            controls,
            valueChange,
            formValues,
            handleSubmit,
            isValid,
            errors,
            deNormalizedScopedSlots,
            normalizedControls,
            formattedOptions,
            onBlur,
            onValidate,
            forceValidation,
            validateAll,
        };
    },
});

function render(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_dynamic_input = vue.resolveComponent("dynamic-input");

  return (vue.openBlock(), vue.createBlock("form", vue.mergeProps({
    class: "dynamic-form",
    novalidate: "",
    id: _ctx.form.id,
    name: _ctx.form.id
  }, _ctx.formattedOptions, {
    onSubmit: _cache[1] || (_cache[1] = vue.withModifiers((...args) => (_ctx.handleSubmit(...args)), ["prevent"]))
  }), [
    (vue.openBlock(true), vue.createBlock(vue.Fragment, null, vue.renderList(_ctx.controls, (control) => {
      return (vue.openBlock(), vue.createBlock(_component_dynamic_input, {
        key: control.name,
        control: control,
        forceValidation: _ctx.forceValidation,
        onChange: _ctx.valueChange,
        onBlur: _ctx.onBlur,
        onValidate: _ctx.onValidate
      }, {
        customField: vue.withCtx((props) => [
          (vue.openBlock(true), vue.createBlock(vue.Fragment, null, vue.renderList(_ctx.deNormalizedScopedSlots, (slot) => {
            return (vue.openBlock(), vue.createBlock("div", {
              key: slot,
              class: "custom-form-wrapper"
            }, [
              (props.control.name === slot)
                ? vue.renderSlot(_ctx.$slots, slot, {
                    key: 0,
                    control: _ctx.normalizedControls[slot],
                    onChange: props.onChange,
                    onBlur: props.onBlur
                  })
                : vue.createCommentVNode("v-if", true)
            ]))
          }), 128 /* KEYED_FRAGMENT */))
        ]),
        _: 2
      }, 1032 /* PROPS, DYNAMIC_SLOTS */, ["control", "forceValidation", "onChange", "onBlur", "onValidate"]))
    }), 128 /* KEYED_FRAGMENT */))
  ], 16 /* FULL_PROPS */, ["id", "name"]))
}

script$7.render = render;
script$7.__file = "src/components/dynamic-form/DynamicForm.vue";

function createDynamicForms(options) {
    const vdf = {
        options,
        install(app) {
            const self = this;
            {
                // Vue 3 only
                console.log('IS VUEEEE 333');
            }
            app.component('dynamic-form', script$7);
            app.provide(dynamicFormsSymbol, self);
        },
    };
    return vdf;
}

const isEmptyInputValue = (value) => value == null || value === '';
const EMAIL_REGEXP = /^(?=.{1,254}$)(?=.{1,64}@)[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+(\.[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+)*@[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?(\.[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?)*$/;
const URL_REGEXP = /^((?:(https?):\/\/)?((?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9])\.(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9])\.)(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9])\.)(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9]))|(?:(?:(?:\w+\.){1,2}[\w]{2,3})))(?::(\d+))?((?:\/[\w]+)*)(?:\/|(\/[\w]+\.[\w]{3,4})|(\?(?:([\w]+=[\w]+)&)*([\w]+=[\w]+))?|\?(?:(wsdl|wadl))))$/;
const required = (value) => ({
    required: isEmptyInputValue(value) ? true : null,
});
const min = (min) => (value) => {
    if (isEmptyInputValue(value) || isEmptyInputValue(min)) {
        return { min: null }; // don't validate empty values to allow optional controls
    }
    const minValue = parseFloat(`${value}`);
    return {
        min: !isNaN(minValue) && minValue < min ? { min, actual: +minValue } : null,
    };
};
const max = (max) => (value) => {
    if (isEmptyInputValue(value) || isEmptyInputValue(max)) {
        return { max: null }; // don't validate empty values to allow optional controls
    }
    const maxValue = parseFloat(`${value}`);
    // Controls with NaN values after parsing should be treated as not having a
    // maximum, per the HTML forms spec: https://www.w3.org/TR/html5/forms.html#attr-input-max
    return {
        max: !isNaN(maxValue) && maxValue < max ? { max, actual: +maxValue } : null,
    };
};
const email = (value) => {
    if (isEmptyInputValue(value)) {
        return { email: null }; // don't validate empty values to allow optional controls
    }
    return { email: EMAIL_REGEXP.test(`${value}`) ? null : true };
};
const url = (value) => {
    if (isEmptyInputValue(value)) {
        return { url: null }; // don't validate empty values to allow optional controls
    }
    return { url: URL_REGEXP.test(`${value}`) ? null : true };
};
const minLength = (minLength) => (value) => {
    if (isEmptyInputValue(value)) {
        return { minLength: null }; // don't validate empty values to allow optional controls
    }
    const length = value ? `${value}`.length : 0;
    return {
        minLength: length < minLength
            ? { requiredLength: minLength, actualLength: length }
            : null,
    };
};
const maxLength = (maxLength) => (value) => {
    if (isEmptyInputValue(value)) {
        return { maxLength: null }; // don't validate empty values to allow optional controls
    }
    const length = value ? `${value}`.length : 0;
    return {
        maxLength: length > maxLength
            ? { requiredLength: maxLength, actualLength: length }
            : null,
    };
};
const pattern = (pattern) => {
    if (!pattern)
        return null;
    let regex;
    let regexStr;
    if (typeof pattern === 'string') {
        regexStr = '';
        if (pattern.charAt(0) !== '^')
            regexStr += '^';
        regexStr += pattern;
        if (pattern.charAt(pattern.length - 1) !== '$')
            regexStr += '$';
        regex = new RegExp(regexStr);
    }
    else {
        regexStr = pattern;
        regex = pattern;
    }
    return (value) => {
        if (isEmptyInputValue(value)) {
            return { pattern: null }; // don't validate empty values to allow optional controls
        }
        const patternValue = value;
        return {
            pattern: regex.test(patternValue)
                ? null
                : { requiredPattern: regexStr, actualValue: value },
        };
    };
};

exports.CheckboxField = CheckboxField;
exports.ColorField = ColorField;
exports.CustomField = CustomField;
exports.EmailField = EmailField;
exports.FieldBase = FieldBase;
exports.FieldControl = FieldControl;
exports.NumberField = NumberField;
exports.PasswordField = PasswordField;
exports.RadioField = RadioField;
exports.SelectField = SelectField;
exports.TextAreaField = TextAreaField;
exports.TextField = TextField;
exports.Validator = Validator;
exports.ValidatorTrigger = ValidatorTrigger;
exports.createDynamicForms = createDynamicForms;
exports.dynamicFormsSymbol = dynamicFormsSymbol;
exports.email = email;
exports.isEmptyInputValue = isEmptyInputValue;
exports.max = max;
exports.maxLength = maxLength;
exports.min = min;
exports.minLength = minLength;
exports.pattern = pattern;
exports.required = required;
exports.url = url;
exports.useDynamicForms = useDynamicForms;
